#define BUILD_NUMBER \
	"build-18557794"
#define BUILD_NUMBER_NUMERIC \
	18557794
#define BUILD_NUMBER_NUMERIC_STRING \
	"18557794"
#define PRODUCT_BUILD_NUMBER \
	"product-build-31214"
#define PRODUCT_BUILD_NUMBER_NUMERIC \
	31214
#define PRODUCT_BUILD_NUMBER_NUMERIC_STRING \
	"31214"
